'use strict'

var INDENT = '    '

function inlineRule (objRule) {
  var str = ''
  Object.keys(objRule).forEach(function (rule) {
    str += rule + ':' + objRule[rule] + ';'
  })
  return str
}

function Stylize (styleFile) {
  function styleClass (cssClass) {
    return 'class="' + cssClass + '"'
  }

  function styleInline (cssClass) {
    return 'style="' + inlineRule(styleFile['.' + cssClass]) + '"'
  }

  if (!styleFile) return styleClass
  return styleInline
}

function type (doc) {
  if (doc === null) return 'null'
  if (Array.isArray(doc)) return 'array'
  if (typeof doc === 'string' && /^https?:/.test(doc)) return 'link'
  if (typeof doc === 'object' && typeof doc.toISOString === 'function') return 'date'

  return typeof doc
}

function escape (str) {
  return str.replace(/</g, '&lt;').replace(/>/g, '&gt;')
}

module.exports = function (doc, className, styleFile) {

  if(doc == "") return doc
  
  var style = Stylize(styleFile)

  var forEach = function (list, start, end, fn) {
    if (!list.length) return start + ' ' + end

    var out = start;

    list.forEach(function (key, i) {
      out += fn(key)
    })

    return out + end
  }

  function visit (obj) {
    if (obj === undefined) return ''

    switch (type(obj)) {
      case 'boolean':
        return '<span ' + style('tjson val bool') + '>' + obj + '</span>'

      case 'number':
        return '<span ' + style('tjson val number') + '>' + obj + '</span>'

      case 'date':
        return '<span class="tjson val string">"' + escape(obj.toISOString()) + '"</span>'

      case 'null':
        return '<span ' + style('tjson val null') + '>null</span>'

      case 'string':
        return '<span ' + style('tjson val string') + '>' + escape(obj) + '</span>'

      case 'link':
        return '<span ' + style('tjson val link') + '>"<a href="' + escape(obj) + '">' + escape(obj) + '</a>"</span>'

      case 'array':
        return forEach(obj, '[', ']', visit)

      case 'object':
        var keys = Object.keys(obj).filter(function (key) {
          return obj[key] !== undefined
        })

        return forEach(keys, '<div class="tjson group">', '</div>', function (key) {
          return '<div class="tjson entry"><span ' + style('tjson key') + '>' + key + '</span> ' + visit(obj[key]) + '</div>'
        })
    }

    return ''
  }

  return '<div ' + style('tjson '+className) + '>' + visit(doc) + '</div>'
}
